@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Tue 26 Aug 2008 07:49:00 JST
+ *  Last-modified: Thu 09 Oct 2008 06:44:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_TOKENIZER_H
@@ -49,69 +49,95 @@ namespace clx {
 		class Type = std::basic_string<char>,
 		class Container = std::vector<Type>
 	>
-	class basic_tokenizer : public Container {
+	class basic_tokenizer {
 	public:
 		typedef TokenizerFunc functor;
-		typedef Type value_type;
 		typedef typename functor::char_type char_type;
 		typedef std::basic_string<char_type> string_type;
 		
+		typedef Type value_type;
+		typedef Container container;
+		typedef typename container::size_type size_type;
+		typedef typename container::const_iterator const_iterator;
+		typedef typename container::iterator iterator;
+		
 		// constructor and destructor
-		explicit basic_tokenizer(const functor& f = functor()) :
-			super(), f_(f) {}
+		explicit basic_tokenizer(functor f = functor()) :
+			v_(), f_(f) {}
 		
 		template <class InputIterator>
-		explicit basic_tokenizer(InputIterator first, InputIterator last, const functor& f = functor()) :
-			super(), f_(f) { this->assign(first, last); }
+		explicit basic_tokenizer(InputIterator first, InputIterator last, functor f = functor()) :
+			v_(), f_(f) { this->assign(first, last); }
 		
-		explicit basic_tokenizer(const string_type& s, const functor& f = functor()) :
-			super(), f_(f) { this->assign(s); }
+		explicit basic_tokenizer(const string_type& s, functor f = functor()) :
+			v_(), f_(f) { this->assign(s); }
 		
-		explicit basic_tokenizer(const char_type* s, const functor& f = functor()) :
-			super(), f_(f) { this->assign(s); }
+		explicit basic_tokenizer(const char_type* s, functor f = functor()) :
+			v_(), f_(f) { this->assign(s); }
 		
 		virtual ~basic_tokenizer() {}
 		
 		//operator
 		void reset() {
+			v_.clear();
 			f_.reset();
-			this->clear();
 		}
 		
 		template <class InputIterator>
-		basic_tokenizer& assign(InputIterator first, InputIterator last) {
+		const container& assign(InputIterator first, InputIterator last) {
 			string_type token;
 			
 			this->reset();
 			while (f_(first, last, token)) {
-				this->insert(this->end(), lexical_cast<value_type>(token));
+				v_.insert(v_.end(), lexical_cast<value_type>(token));
 				if (first == last) break;
 				++first;
 			}
-			return *this;
+			return v_;
 		}
 		
-		basic_tokenizer& assign(const string_type& s) {
+		const container& assign(const string_type& s) {
 			typename string_type::const_iterator pos = s.begin();
 			return this->assign(pos, s.end());
 		}
 		
-		basic_tokenizer& assign(const char_type* s) {
+		const container& assign(const char_type* s) {
 			string_type tmp(s);
 			return this->assign(tmp);
 		}
 		
 		template <class InputIterator>
-		basic_tokenizer& operator()(InputIterator first, InputIterator last) {
+		const container& operator()(InputIterator first, InputIterator last) {
 			return this->assign(first, last);
 		}
 		
-		basic_tokenizer& operator()(const string_type& s) { return this->assign(s); }
-		basic_tokenizer& operator()(const char_type* s) { return this->assign(s); }
+		const container& operator()(const string_type& s) { return this->assign(s); }
+		const container& operator()(const char_type* s) { return this->assign(s); }
+		
+		// inherit container methods
+		const value_type& operator[](size_type pos) const { return v_[pos]; }
+		const value_type& at(size_type pos) const { return v_.at(pos); }
+		const value_type& front() const { return v_.front(); }
+		const value_type& back() const { return v_.back(); }
+		
+		bool empty() const { return v_.empty(); }
+		size_type size() const { return v_.size(); }
+		
+		const_iterator begin() const { return v_.begin(); }
+		const_iterator end() const { return v_.end(); }
+		const_iterator rbegin() const { return v_.rbegin(); }
+		const_iterator rend() const { return v_.rend(); }
+		
+		iterator begin() { return v_.begin(); }
+		iterator end() { return v_.end(); }
+		iterator rbegin() { return v_.rbegin(); }
+		iterator rend() { return v_.rend(); }
+		
+		// get container
+		const container& data() const { return v_; }
 		
 	private:
-		typedef Container super;
-		
+		container v_;
 		functor f_;
 	};
 	
@@ -134,7 +160,7 @@ namespace clx {
 	 */
 	/* --------------------------------------------------------------------- */
 	template <class Type, class TokenizerFunc>
-	basic_tokenizer<TokenizerFunc, Type> create_tokenizer(TokenizerFunc f) {
+	inline basic_tokenizer<TokenizerFunc, Type> create_tokenizer(TokenizerFunc f) {
 		return basic_tokenizer<TokenizerFunc, Type>(f);
 	}
 }
